/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * This dmraid code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * dmraid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this dmraid library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

/* Cheers utils.h */

#ifndef _BYTEORDER_H
#define _BYTEORDER_H

static inline int32_t swap32(int32_t x)
{
	return((((u_int32_t)x & 0x000000ffU) << 24) | 
	       (((u_int32_t)x & 0x0000ff00U) <<  8) | 
	       (((u_int32_t)x & 0x00ff0000U) >>  8) | 
	       (((u_int32_t)x & 0xff000000U) >> 24));
}

static inline int16_t swap16(int16_t x)
{
	return((((u_int16_t)x & 0x00ffU) << 8) | 
	       (((u_int16_t)x & 0xff00U) >> 8));
}

#if BYTE_ORDER == LITTLE_ENDIAN

static inline int32_t be32_to_cpu(int32_t x)
{
	return(swap32(x));
}

static inline int16_t be16_to_cpu(int16_t x)
{
	return(swap16(x));
}

static inline int32_t le32_to_cpu(int32_t x)
{
	return(x);
}

static inline int16_t le16_to_cpu(int16_t x)
{
	return(x);
}

#else

static inline int32_t be32_to_cpu(int32_t x)
{
	return(x);
}

static inline int16_t be16_to_cpu(int16_t x)
{
	return(x);
}

static inline int32_t le32_to_cpu(int32_t x)
{
	return(swap32(x));
}

static inline int16_t le16_to_cpu(int16_t x)
{
	return(swap16(x));
}

#endif /* #if BYTE_ORDER == LITTLE_ENDIAN */

static inline int32_t cpu_to_be32(int32_t x)
{
	return(be32_to_cpu(x));
}

static inline int32_t cpu_to_le32(int32_t x)
{
	return(le32_to_cpu(x));
}

static inline int16_t cpu_to_be16(int16_t x)
{
	return(be16_to_cpu(x));
}

static inline int16_t cpu_to_le16(int16_t x)
{
	return(le16_to_cpu(x));
}

#endif
