/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * This dmraid code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * dmraid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this dmraid library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

#include <errno.h>
#include <sys/file.h>
#include "internal.h"
#include "locking.h"

static const char *_lock_dir = "/var/lock/dmraid";
static const char *_lock_file = "/var/lock/dmraid/.lock";

/* flock file. */
static int lock_file(struct lib_context *lc, struct resource *res)
{
	int ret = 0;

	if (lc->lock.lf > -1)
		return 1;

	log_info("locking %s", _lock_file);
	if (lc->lock.lf > -1)
		close(lc->lock.lf);

	if ((lc->lock.lf = open(_lock_file, O_CREAT | O_APPEND | O_RDWR, 0777))
	    < 0) {
		log_err("opening lockfile %s", _lock_file);
		goto out;
	}

	if (flock(lc->lock.lf, LOCK_EX)) {
		log_err("flock lockfile %s", _lock_file);
		close(lc->lock.lf);
		lc->lock.lf = -1;
		goto out;
	}

	ret = 1;

  out:
	return ret;
}

/* unlock file. */
static void unlock_file(struct lib_context *lc, struct resource *res)
{
	if (lc->lock.lf == -1)
		return;

	log_info("unlocking %s", _lock_file);
	unlink(_lock_file);
	if (flock(lc->lock.lf, LOCK_NB | LOCK_UN))
		log_err("flock lockfile %s", _lock_file);

	if (close(lc->lock.lf))
		log_err("close lockfile %s", _lock_file);

	lc->lock.lf = -1;
}

/* File base locking. */
static struct locking file_locking = {
	.lf	= -1,
	.lock 	= lock_file,
	.unlock	= unlock_file,
};

/* Initialize locking. */
int init_locking(struct lib_context *lc)
{
	if (!mk_dir(lc, _lock_dir))
		return 0;

	/* Fail on read-only file system. */
	if (access(_lock_dir, R_OK | W_OK | X_OK) && errno == EROFS)
		return 0;

	lc->lock = file_locking;

	return 1;
}

/* Hide locking. */
int lock_resource(struct lib_context *lc, struct resource *res)
{
	return lc->lock.lock(lc, res);
}

/* Hide unlocking. */
void unlock_resource(struct lib_context *lc, struct resource *res)
{
	 lc->lock.unlock(lc, res);
}
