/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * This dmraid code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * dmraid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this dmraid library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

#include <stdarg.h>
#include "internal.h"

static const char *_prefixes[] = {
	NULL,
	"INFO",
	"NOTICE",
	"WARNING",
	"DEBUG",
	"ERROR",
	"FATAL",
};

static const char *_prefix(int level)
{
	return level < ARRAY_SIZE(_prefixes) ? _prefixes[level] : "UNDEFINED";
}

void plog(struct lib_context *lc, int level, int lf, const char *file,
	  int line, const char *format, ...)
{
	int o = LC_VERBOSE, l = level;
	FILE *f = stdout;
	va_list ap;

 	if (level == _PLOG_DEBUG) {
		o = LC_DEBUG;
		l -= _PLOG_WARN;
	}

	if (level == _PLOG_ERR || level == _PLOG_FATAL)
		f = stderr;
	else if (lc_opt(lc, o) < l)
		return;

	if (_prefix(level))
		fprintf(f, "%s: ", _prefix(level));

	va_start(ap, format);
	vfprintf(f, format, ap);
	va_end(ap);

	fputc('\n', f);
}
