/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * This dmraid code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * dmraid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this dmraid library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

#ifndef _LOG_H_
#define _LOG_H_

#include <stdio.h>

/* Compatibility type for logging. */
#define	PRIzu	"zu"

/* Log levels. */
#define _PLOG_INFO 1
#define _PLOG_NOTICE 2
#define _PLOG_WARN 3
#define _PLOG_DEBUG 4
#define _PLOG_ERR 5
#define _PLOG_FATAL 6

struct lib_context;

void plog(struct lib_context *lc, int level, int lf, const char *file,
	  int line, const char *format, ...);

#  ifdef DMRAID_MINI

#define log_info(x...)
#define log_notice(x...)
#define log_warn(x...)
#define log_debug(x...)
#define log_dbg(x...)

#  else

#define _log_info(lf, x...) plog(lc, _PLOG_INFO, lf, __FILE__, __LINE__, x)
#define log_info(x...) _log_info(1, x)

#define _log_notice(lf, x...) plog(lc, _PLOG_NOTICE, lf, __FILE__, __LINE__, x)
#define log_notice(x...) _log_notice(1, x)

#define _log_warn(lf, x...) plog(lc, _PLOG_WARN, lf, __FILE__, __LINE__, x)
#define log_warn(x...) _log_warn(1, x)

#define _log_debug(lf, x...) plog(lc, _PLOG_DEBUG, lf, __FILE__, __LINE__, x)
#define log_debug(x...) _log_debug(1, x)
#define log_dbg(x...) log_debug(x)

#  endif

#define _log_error(lf, x...) plog(lc, _PLOG_ERR, lf, __FILE__, __LINE__, x)
#define log_error(x...) _log_error(1, x)
#define log_err(x...) log_error(x)

#define _log_fatal(lf, x...) plog(lc, _PLOG_FATAL, lf, __FILE__, __LINE__, x)
#define log_fatal(x...) _log_fatal(1, x)


#define	LOG_ERR(ret, x...)	do { log_err(x); return ret; } while (0)
#define _log_print(lf, x...)	plog(lc, 0, lf, __FILE__, __LINE__, x)
#define log_print(x...)	_log_print(1, x)
#define log_print_nnl(x...)	_log_print(0, x)
#define	LOG_PRINT(ret, x...)	do { log_print(x); return ret; } while (0)
#define	LOG_PRINT_NNL(ret, x...) do { _log_print(0, x); return ret; } while (0)

#endif
