/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * This dmraid code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * dmraid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this dmraid library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

#ifndef _LIB_CONTEXT_H_
#define _LIB_CONTEXT_H_

#include "datastruct/list.h"
#include "locking/locking.h"
#include "misc/misc.h"

enum lc_lists {
	LC_FORMATS,	/* Metadata format handlers. */
	LC_DISK_INFOS,	/* Disks dioscovered. */
	LC_RAID_SETS,	/* Raid sets grouped. */
	/* Add new lists below here ! */
	LC_LISTS_SIZE,	/* Must be the last enumerator. */
};

enum lc_options {
	LC_COLUMN = 0,
	LC_DEBUG,
	LC_DUMP,
	LC_GROUP,
	LC_TEST,
	LC_VERBOSE,	/* Add new options below this one ! */
	LC_OPTIONS_SIZE,/* Must be the last enumerator. */
};

struct lib_context {
	struct lib_version version;
	char *cmd;

	/* Various option conters used throughout the library. */
	int opts[LC_OPTIONS_SIZE];

	/*
	 * Lists for:
	 *
	 *	o metadata format handlers the library supports
	 * 	o block devices discovered
	 * 	o RAID devices discovered
	 * 	o RAID sets grouped
	 */
	struct list_head lists[LC_LISTS_SIZE];

	struct	locking lock;
	mode_t	mode;
};

struct lib_context *alloc_lib_context(char **argv);
void free_lib_context(struct lib_context *lc);
int lc_opt(struct lib_context *lc, enum lc_options o);
int lc_inc_opt(struct lib_context *lc, int o);
struct list_head *lc_list(struct lib_context *lc, int l);

#endif
